/* 
 * This file is adapted to be compatible with
 * pcc32 C compiler and Windows 32 bit.
 */

#ifndef _INC_STDIO
#define _INC_STDIO

#pragma pack(push,8) // 8 byte alignment.

#ifndef _SIZE_T_DEFINED
typedef unsigned int size_t;
#define _SIZE_T_DEFINED
#endif

#ifndef _WCHAR_T_DEFINED
typedef unsigned short wchar_t;
#define _WCHAR_T_DEFINED
#endif

#ifndef _WCTYPE_T_DEFINED
typedef wchar_t wint_t;
typedef wchar_t wctype_t;
#define _WCTYPE_T_DEFINED
#endif

#ifndef _VA_LIST_DEFINED
typedef char *  va_list;
#define _VA_LIST_DEFINED
#endif

#define BUFSIZ  512
#define _NFILE      _NSTREAM_
#define _NSTREAM_   512
#define _IOB_ENTRIES 20

#define EOF     (-1)

#ifndef _FILE_DEFINED

struct _iobuf {
        char *_ptr;
        int   _cnt;
        char *_base;
        int   _flag;
        int   _file;
        int   _charbuf;
        int   _bufsiz;
        char *_tmpfname;
        };
typedef struct _iobuf FILE;
#define _FILE_DEFINED
#endif

#define _P_tmpdir   "\\"
#define _wP_tmpdir  L"\\"

#define L_tmpnam sizeof(_P_tmpdir)+12
#define SEEK_CUR    1
#define SEEK_END    2
#define SEEK_SET    0
#define FILENAME_MAX    260
#define FOPEN_MAX       20
#define _SYS_OPEN       20
#define TMP_MAX         32767

#ifndef NULL
#define NULL    ((void *)0)
#endif

extern FILE _iob[];

#ifndef _FPOS_T_DEFINED
typedef long fpos_t;
#define _FPOS_T_DEFINED
#endif

#if 1
extern FILE (*_imp___iob)[];
#define _iob    (*_imp___iob)
#define stdin  (&_iob[0])
#define stdout (&_iob[1])
#define stderr (&_iob[2])
#else
extern FILE *stdin;
extern FILE *stdout;
extern FILE *stderr;
#endif

#define _IOREAD         0x0001
#define _IOWRT          0x0002
#define _IOFBF          0x0000
#define _IOLBF          0x0040
#define _IONBF          0x0004
#define _IOMYBUF        0x0008
#define _IOEOF          0x0010
#define _IOERR          0x0020
#define _IOSTRG         0x0040
#define _IORW           0x0080
#define _IOAPPEND       0x0200

#ifndef _STDIO_DEFINED

int _filbuf(FILE *);
int _flsbuf(int, FILE *);

FILE * _fsopen(const char *, const char *, int);
void clearerr(FILE *);
int fclose(FILE *);
int _fcloseall(void);
FILE * _fdopen(int, const char *);
int feof(FILE *);
int ferror(FILE *);
int fflush(FILE *);
int fgetc(FILE *);
int _fgetchar(void);
int fgetpos(FILE *, fpos_t *);
char * fgets(char *, int, FILE *);
int _fileno(FILE *);
int _flushall(void);
FILE * fopen(const char *, const char *);
int fprintf(FILE *, const char *, ...);
int fputc(int, FILE *);
int _fputchar(int);
int fputs(const char *, FILE *);
size_t fread(void *, size_t, size_t, FILE *);
FILE * freopen(const char *, const char *, FILE *);
int fscanf(FILE *, const char *, ...);
int fsetpos(FILE *, const fpos_t *);
int fseek(FILE *, long, int);
long ftell(FILE *);
size_t fwrite(const void *, size_t, size_t, FILE *);
int getc(FILE *);
int getchar(void);
int _getmaxstdio(void);
char * gets(char *);
int _getw(FILE *);
void perror(const char *);
int _pclose(FILE *);
FILE * _popen(const char *, const char *);
int printf(const char *, ...);
int putc(int, FILE *);
int putchar(int);
int puts(const char *);
int _putw(int, FILE *);
int remove(const char *);
int rename(const char *, const char *);
void rewind(FILE *);
int _rmtmp(void);
int scanf(const char *, ...);
void setbuf(FILE *, char *);
int _setmaxstdio(int);
int setvbuf(FILE *, char *, int, size_t);
int _snprintf(char *, size_t, const char *, ...);
int sprintf(char *, const char *, ...);
int sscanf(const char *, const char *, ...);
char * _tempnam(const char *, const char *);
FILE * tmpfile(void);
char * tmpnam(char *);
int ungetc(int, FILE *);
int _unlink(const char *);
int vfprintf(FILE *, const char *, va_list);
int vprintf(const char *, va_list);
int _vsnprintf(char *, size_t, const char *, va_list);
int vsprintf(char *, const char *, va_list);

#ifndef _WSTDIO_DEFINED

#ifndef WEOF
#define WEOF (wint_t)(0xFFFF)
#endif

FILE * _wfsopen(const wchar_t *, const wchar_t *, int);
wint_t fgetwc(FILE *);
wint_t _fgetwchar(void);
wint_t fputwc(wint_t, FILE *);
wint_t _fputwchar(wint_t);
wint_t getwc(FILE *);
wint_t getwchar(void);
wint_t putwc(wint_t, FILE *);
wint_t putwchar(wint_t);
wint_t ungetwc(wint_t, FILE *);

wchar_t * fgetws(wchar_t *, int, FILE *);
int fputws(const wchar_t *, FILE *);
wchar_t * _getws(wchar_t *);
int _putws(const wchar_t *);

int fwprintf(FILE *, const wchar_t *, ...);
int wprintf(const wchar_t *, ...);
int _snwprintf(wchar_t *, size_t, const wchar_t *, ...);
int swprintf(wchar_t *, const wchar_t *, ...);
int vfwprintf(FILE *, const wchar_t *, va_list);
int vwprintf(const wchar_t *, va_list);
int _vsnwprintf(wchar_t *, size_t, const wchar_t *, va_list);
int vswprintf(wchar_t *, const wchar_t *, va_list);
int fwscanf(FILE *, const wchar_t *, ...);
int swscanf(const wchar_t *, const wchar_t *, ...);
int wscanf(const wchar_t *, ...);

#define getwchar()              fgetwc(stdin)
#define putwchar(_c)            fputwc((_c),stdout)
#define getwc(_stm)             fgetwc(_stm)
#define putwc(_c,_stm)          fputwc(_c,_stm)

FILE * _wfdopen(int, const wchar_t *);
FILE * _wfopen(const wchar_t *, const wchar_t *);
FILE * _wfreopen(const wchar_t *, const wchar_t *, FILE *);
void _wperror(const wchar_t *);
FILE * _wpopen(const wchar_t *, const wchar_t *);
int _wremove(const wchar_t *);
wchar_t * _wtempnam(const wchar_t *, const wchar_t *);
wchar_t * _wtmpnam(wchar_t *);


#define _WSTDIO_DEFINED
#endif  /* _WSTDIO_DEFINED */

#define _STDIO_DEFINED
#endif  /* _STDIO_DEFINED */

#define feof(_stream)     ((_stream)->_flag & _IOEOF)
#define ferror(_stream)   ((_stream)->_flag & _IOERR)
#define _fileno(_stream)  ((_stream)->_file)
#define getc(_stream)     (--(_stream)->_cnt >= 0 \
                ? 0xff & *(_stream)->_ptr++ : _filbuf(_stream))
#define putc(_c,_stream)  (--(_stream)->_cnt >= 0 \
                ? 0xff & (*(_stream)->_ptr++ = (char)(_c)) :  _flsbuf((_c),(_stream)))
#define getchar()         getc(stdin)
#define putchar(_c)       putc((_c),stdout)

#ifdef  _MT
#undef  getc
#undef  putc
#undef  getchar
#undef  putchar
#endif

#if     !__STDC__

#define P_tmpdir  _P_tmpdir
#define SYS_OPEN  _SYS_OPEN

int fcloseall(void);
FILE * fdopen(int, const char *);
int fgetchar(void);
int fileno(FILE *);
int flushall(void);
int fputchar(int);
int getw(FILE *);
int putw(int, FILE *);
int rmtmp(void);
char * tempnam(const char *, const char *);
int unlink(const char *);

#endif  /* __STDC__ */

#endif	/* _INC_STDIO */
